"use strict";
/**
 * @fileoverview Interactions should be awaited
 * @author Yann Braga
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
//------------------------------------------------------------------------------
// Requirements
//------------------------------------------------------------------------------
const ts_dedent_1 = __importDefault(require("ts-dedent"));
const await_interactions_1 = __importDefault(require("../../../lib/rules/await-interactions"));
const rule_tester_1 = __importDefault(require("../../utils/rule-tester"));
//------------------------------------------------------------------------------
// Tests
//------------------------------------------------------------------------------
rule_tester_1.default.run('await-interactions', await_interactions_1.default, {
    valid: [
        (0, ts_dedent_1.default) `
      Basic.play = async () => {
        await userEvent.click(button)
      }
    `,
        (0, ts_dedent_1.default) `
      WithModalOpen.play = ({ canvasElement }) => {
        const MyButton = canvas.getByRole('button')
      }
    `,
        (0, ts_dedent_1.default) `
      WithModalOpen.play = async ({ canvasElement }) => {
        const MyButton = await canvas.findByRole('button')
      }
    `,
        (0, ts_dedent_1.default) `
      WithModalOpen.play = async ({ canvasElement }) => {
        const element: HTMLButtonElement = await within(canvasElement).findByText(/Hello/i)
        await userEvent.click(element, undefined, { clickCount: 2 })
        await userEvent.click(await within(canvasElement).findByText(/Hello/i), undefined, {
          clickCount: 2,
        })
      }
    `,
        (0, ts_dedent_1.default) `
      export const WithModalOpen = {
        play: async ({ canvasElement }) => {
          const element: HTMLButtonElement = await within(canvasElement).findByText(/Hello/i)
          await userEvent.click(element, undefined, { clickCount: 2 })
          await userEvent.click(await within(canvasElement).findByText(/Hello/i), undefined, {
            clickCount: 2,
          })
        }
      }
    `,
        'await expect(foo).toBe(bar)',
        (0, ts_dedent_1.default) `
      Basic.play = async () => {
        await waitForElementToBeRemoved(() => canvas.findByText('Loading...'))
        await waitForElementToBeRemoved(() => userEvent.hover(canvas.getByTestId('password-error-info')))
      }
    `,
        (0, ts_dedent_1.default) `
      import { userEvent } from '../utils'
      import { within } from '@storybook/testing-library'

      Basic.play = async (context) => {
        const canvas = within(context)
        userEvent.click(canvas.getByRole('button'))
      }
    `,
        (0, ts_dedent_1.default) `
      Basic.play = async () => {
        const userEvent = { test: () => {} }
        // should not complain
        userEvent.test()
      }
    `,
        // // @TODO: https://github.com/storybookjs/eslint-plugin-storybook/issues/28
        // dedent`
        //   Block.parameters = {
        //     async puppeteerTest(page) {
        //       const element = await page.$('[data-test-block]');
        //       await element.hover();
        //       const textContent = await element.getProperty('textContent');
        //       const text = await textContent.jsonValue();
        //       expect(text).toBe('I am hovered');
        //     },
        //   };
        // `,
        'Basic.play = async () => userEvent.click(button)',
        'Basic.play = async () => { return userEvent.click(button) }',
        (0, ts_dedent_1.default) `
      export const SecondStory = {
        play: async (context) => {
          await FirstStory.play(context)
        }
      }
    `,
        (0, ts_dedent_1.default) `
      export const SecondStory = {
        play: async (context) => {
          await FirstStory.play?.(context)
        }
      }
    `,
        (0, ts_dedent_1.default) `
      export const SecondStory = {
        play: async (context) => {
          await FirstStory.play!(context)
        }
      }
    `,
    ],
    invalid: [
        {
            code: (0, ts_dedent_1.default) `
        import { expect } from '@storybook/jest'
        WithModalOpen.play = async ({ args }) => {
          // should complain
          expect(args.onClick).toHaveBeenCalled()
        }
      `,
            output: (0, ts_dedent_1.default) `
        import { expect } from '@storybook/jest'
        WithModalOpen.play = async ({ args }) => {
          // should complain
          await expect(args.onClick).toHaveBeenCalled()
        }
      `,
            errors: [
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'toHaveBeenCalled' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
                import { expect } from '@storybook/jest'
                WithModalOpen.play = async ({ args }) => {
                  // should complain
                  await expect(args.onClick).toHaveBeenCalled()
                }
              `,
                        },
                    ],
                },
            ],
        },
        {
            code: (0, ts_dedent_1.default) `
        WithModalOpen.play = ({ canvasElement }) => {
          const canvas = within(canvasElement)

          const foodItem = canvas.findByText(/Cheeseburger/i)
        }
      `,
            output: (0, ts_dedent_1.default) `
        WithModalOpen.play = async ({ canvasElement }) => {
          const canvas = within(canvasElement)

          const foodItem = await canvas.findByText(/Cheeseburger/i)
        }
      `,
            errors: [
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'findByText' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
                WithModalOpen.play = async ({ canvasElement }) => {
                  const canvas = within(canvasElement)

                  const foodItem = await canvas.findByText(/Cheeseburger/i)
                }
              `,
                        },
                    ],
                },
            ],
        },
        {
            code: (0, ts_dedent_1.default) `
        WithModalOpen.play = async ({ canvasElement }) => {
          const canvas = within(canvasElement)
          const foodItem = canvas.findByText(/Cheeseburger/i)
          userEvent.click(foodItem)
          const modalButton = canvas.findByLabelText('increase quantity by one')
          userEvent.click(modalButton)
        }
      `,
            output: (0, ts_dedent_1.default) `
        WithModalOpen.play = async ({ canvasElement }) => {
          const canvas = within(canvasElement)
          const foodItem = await canvas.findByText(/Cheeseburger/i)
          await userEvent.click(foodItem)
          const modalButton = await canvas.findByLabelText('increase quantity by one')
          await userEvent.click(modalButton)
        }
      `,
            errors: [
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'findByText' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
                WithModalOpen.play = async ({ canvasElement }) => {
                  const canvas = within(canvasElement)
                  const foodItem = await canvas.findByText(/Cheeseburger/i)
                  userEvent.click(foodItem)
                  const modalButton = canvas.findByLabelText('increase quantity by one')
                  userEvent.click(modalButton)
                }
            `,
                        },
                    ],
                },
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'userEvent' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
                WithModalOpen.play = async ({ canvasElement }) => {
                  const canvas = within(canvasElement)
                  const foodItem = canvas.findByText(/Cheeseburger/i)
                  await userEvent.click(foodItem)
                  const modalButton = canvas.findByLabelText('increase quantity by one')
                  userEvent.click(modalButton)
                }
               `,
                        },
                    ],
                },
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'findByLabelText' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
                WithModalOpen.play = async ({ canvasElement }) => {
                  const canvas = within(canvasElement)
                  const foodItem = canvas.findByText(/Cheeseburger/i)
                  userEvent.click(foodItem)
                  const modalButton = await canvas.findByLabelText('increase quantity by one')
                  userEvent.click(modalButton)
                }
               `,
                        },
                    ],
                },
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'userEvent' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
                WithModalOpen.play = async ({ canvasElement }) => {
                  const canvas = within(canvasElement)
                  const foodItem = canvas.findByText(/Cheeseburger/i)
                  userEvent.click(foodItem)
                  const modalButton = canvas.findByLabelText('increase quantity by one')
                  await userEvent.click(modalButton)
                }
               `,
                        },
                    ],
                },
            ],
        },
        {
            code: (0, ts_dedent_1.default) `
          WithModalOpen.play = async ({ canvasElement }) => {
            const element: HTMLButtonElement = within(canvasElement).findByText(/Hello/i)
            userEvent.click(element, undefined, { clickCount: 2 })
            userEvent.click(within(canvasElement).findByText(/Hello/i), undefined, {
              clickCount: 2,
            })
          }
        `,
            output: (0, ts_dedent_1.default) `
        WithModalOpen.play = async ({ canvasElement }) => {
          const element: HTMLButtonElement = await within(canvasElement).findByText(/Hello/i)
          await userEvent.click(element, undefined, { clickCount: 2 })
          await userEvent.click(await within(canvasElement).findByText(/Hello/i), undefined, {
            clickCount: 2,
          })
        }
      `,
            errors: [
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'findByText' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
                WithModalOpen.play = async ({ canvasElement }) => {
                  const element: HTMLButtonElement = await within(canvasElement).findByText(/Hello/i)
                  userEvent.click(element, undefined, { clickCount: 2 })
                  userEvent.click(within(canvasElement).findByText(/Hello/i), undefined, {
                    clickCount: 2,
                  })
                }
              `,
                        },
                    ],
                },
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'userEvent' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
                WithModalOpen.play = async ({ canvasElement }) => {
                  const element: HTMLButtonElement = within(canvasElement).findByText(/Hello/i)
                  await userEvent.click(element, undefined, { clickCount: 2 })
                  userEvent.click(within(canvasElement).findByText(/Hello/i), undefined, {
                    clickCount: 2,
                  })
                }
              `,
                        },
                    ],
                },
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'userEvent' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
                WithModalOpen.play = async ({ canvasElement }) => {
                  const element: HTMLButtonElement = within(canvasElement).findByText(/Hello/i)
                  userEvent.click(element, undefined, { clickCount: 2 })
                  await userEvent.click(within(canvasElement).findByText(/Hello/i), undefined, {
                    clickCount: 2,
                  })
                }
              `,
                        },
                    ],
                },
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'findByText' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
                WithModalOpen.play = async ({ canvasElement }) => {
                  const element: HTMLButtonElement = within(canvasElement).findByText(/Hello/i)
                  userEvent.click(element, undefined, { clickCount: 2 })
                  userEvent.click(await within(canvasElement).findByText(/Hello/i), undefined, {
                    clickCount: 2,
                  })
                }
              `,
                        },
                    ],
                },
            ],
        },
        {
            code: (0, ts_dedent_1.default) `
        export const WithModalOpen = {
          play: async ({ canvasElement, args }) => {
            const element: HTMLButtonElement = within(canvasElement).findByText(/Hello/i)
            userEvent.click(element, undefined, { clickCount: 2 })
            userEvent.click(within(canvasElement).findByText(/Hello/i), undefined, {
              clickCount: 2,
            })
            expect(args.onSubmit).toHaveBeenCalled()
          }
        }
      `,
            output: (0, ts_dedent_1.default) `
        export const WithModalOpen = {
          play: async ({ canvasElement, args }) => {
            const element: HTMLButtonElement = await within(canvasElement).findByText(/Hello/i)
            await userEvent.click(element, undefined, { clickCount: 2 })
            await userEvent.click(await within(canvasElement).findByText(/Hello/i), undefined, {
              clickCount: 2,
            })
            await expect(args.onSubmit).toHaveBeenCalled()
          }
        }
      `,
            errors: [
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'findByText' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
              export const WithModalOpen = {
                play: async ({ canvasElement, args }) => {
                  const element: HTMLButtonElement = await within(canvasElement).findByText(/Hello/i)
                  userEvent.click(element, undefined, { clickCount: 2 })
                  userEvent.click(within(canvasElement).findByText(/Hello/i), undefined, {
                    clickCount: 2,
                  })
                  expect(args.onSubmit).toHaveBeenCalled()
                }
              }
            `,
                        },
                    ],
                },
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'userEvent' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
              export const WithModalOpen = {
                play: async ({ canvasElement, args }) => {
                  const element: HTMLButtonElement = within(canvasElement).findByText(/Hello/i)
                  await userEvent.click(element, undefined, { clickCount: 2 })
                  userEvent.click(within(canvasElement).findByText(/Hello/i), undefined, {
                    clickCount: 2,
                  })
                  expect(args.onSubmit).toHaveBeenCalled()
                }
              }
            `,
                        },
                    ],
                },
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'userEvent' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
              export const WithModalOpen = {
                play: async ({ canvasElement, args }) => {
                  const element: HTMLButtonElement = within(canvasElement).findByText(/Hello/i)
                  userEvent.click(element, undefined, { clickCount: 2 })
                  await userEvent.click(within(canvasElement).findByText(/Hello/i), undefined, {
                    clickCount: 2,
                  })
                  expect(args.onSubmit).toHaveBeenCalled()
                }
              }
            `,
                        },
                    ],
                },
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'findByText' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
              export const WithModalOpen = {
                play: async ({ canvasElement, args }) => {
                  const element: HTMLButtonElement = within(canvasElement).findByText(/Hello/i)
                  userEvent.click(element, undefined, { clickCount: 2 })
                  userEvent.click(await within(canvasElement).findByText(/Hello/i), undefined, {
                    clickCount: 2,
                  })
                  expect(args.onSubmit).toHaveBeenCalled()
                }
              }
            `,
                        },
                    ],
                },
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'toHaveBeenCalled' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
              export const WithModalOpen = {
                play: async ({ canvasElement, args }) => {
                  const element: HTMLButtonElement = within(canvasElement).findByText(/Hello/i)
                  userEvent.click(element, undefined, { clickCount: 2 })
                  userEvent.click(within(canvasElement).findByText(/Hello/i), undefined, {
                    clickCount: 2,
                  })
                  await expect(args.onSubmit).toHaveBeenCalled()
                }
              }
            `,
                        },
                    ],
                },
            ],
        },
        {
            code: (0, ts_dedent_1.default) `
        export const AfterLoadingState = {
          play: async ({ canvasElement, args }) => {
            const canvas = within(canvasElement)
            waitForElementToBeRemoved(async () => {
              canvas.findByText('Loading...')
            }, { timeout: 2000 })
            const button = canvas.findByText('Loaded!')
            userEvent.click(button)
            waitFor(async () => {
              expect(args.onSubmit).toHaveBeenCalled()
            })
          }
        }
      `,
            output: (0, ts_dedent_1.default) `
        export const AfterLoadingState = {
          play: async ({ canvasElement, args }) => {
            const canvas = within(canvasElement)
            await waitForElementToBeRemoved(async () => {
              await canvas.findByText('Loading...')
            }, { timeout: 2000 })
            const button = await canvas.findByText('Loaded!')
            await userEvent.click(button)
            await waitFor(async () => {
              await expect(args.onSubmit).toHaveBeenCalled()
            })
          }
        }
      `,
            errors: [
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'waitForElementToBeRemoved' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
              export const AfterLoadingState = {
                play: async ({ canvasElement, args }) => {
                  const canvas = within(canvasElement)
                  await waitForElementToBeRemoved(async () => {
                    canvas.findByText('Loading...')
                  }, { timeout: 2000 })
                  const button = canvas.findByText('Loaded!')
                  userEvent.click(button)
                  waitFor(async () => {
                    expect(args.onSubmit).toHaveBeenCalled()
                  })
                }
              }
            `,
                        },
                    ],
                },
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'findByText' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
              export const AfterLoadingState = {
                play: async ({ canvasElement, args }) => {
                  const canvas = within(canvasElement)
                  waitForElementToBeRemoved(async () => {
                    await canvas.findByText('Loading...')
                  }, { timeout: 2000 })
                  const button = canvas.findByText('Loaded!')
                  userEvent.click(button)
                  waitFor(async () => {
                    expect(args.onSubmit).toHaveBeenCalled()
                  })
                }
              }
            `,
                        },
                    ],
                },
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'findByText' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
              export const AfterLoadingState = {
                play: async ({ canvasElement, args }) => {
                  const canvas = within(canvasElement)
                  waitForElementToBeRemoved(async () => {
                    canvas.findByText('Loading...')
                  }, { timeout: 2000 })
                  const button = await canvas.findByText('Loaded!')
                  userEvent.click(button)
                  waitFor(async () => {
                    expect(args.onSubmit).toHaveBeenCalled()
                  })
                }
              }
            `,
                        },
                    ],
                },
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'userEvent' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
              export const AfterLoadingState = {
                play: async ({ canvasElement, args }) => {
                  const canvas = within(canvasElement)
                  waitForElementToBeRemoved(async () => {
                    canvas.findByText('Loading...')
                  }, { timeout: 2000 })
                  const button = canvas.findByText('Loaded!')
                  await userEvent.click(button)
                  waitFor(async () => {
                    expect(args.onSubmit).toHaveBeenCalled()
                  })
                }
              }
            `,
                        },
                    ],
                },
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'waitFor' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
              export const AfterLoadingState = {
                play: async ({ canvasElement, args }) => {
                  const canvas = within(canvasElement)
                  waitForElementToBeRemoved(async () => {
                    canvas.findByText('Loading...')
                  }, { timeout: 2000 })
                  const button = canvas.findByText('Loaded!')
                  userEvent.click(button)
                  await waitFor(async () => {
                    expect(args.onSubmit).toHaveBeenCalled()
                  })
                }
              }
            `,
                        },
                    ],
                },
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'toHaveBeenCalled' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
              export const AfterLoadingState = {
                play: async ({ canvasElement, args }) => {
                  const canvas = within(canvasElement)
                  waitForElementToBeRemoved(async () => {
                    canvas.findByText('Loading...')
                  }, { timeout: 2000 })
                  const button = canvas.findByText('Loaded!')
                  userEvent.click(button)
                  waitFor(async () => {
                    await expect(args.onSubmit).toHaveBeenCalled()
                  })
                }
              }
            `,
                        },
                    ],
                },
            ],
        },
        {
            code: (0, ts_dedent_1.default) `
        export const FourthStory = {
          play: async (context) => {
            FirstStory.play(context)
            SecondStory.play!(context)
            ThirdStory.play?.(context)
          }
        }
      `,
            output: (0, ts_dedent_1.default) `
        export const FourthStory = {
          play: async (context) => {
            await FirstStory.play(context)
            await SecondStory.play!(context)
            await ThirdStory.play?.(context)
          }
        }
      `,
            errors: [
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'play' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
                export const FourthStory = {
                  play: async (context) => {
                    await FirstStory.play(context)
                    SecondStory.play!(context)
                    ThirdStory.play?.(context)
                  }
                }
              `,
                        },
                    ],
                },
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'play' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
                export const FourthStory = {
                  play: async (context) => {
                    FirstStory.play(context)
                    await SecondStory.play!(context)
                    ThirdStory.play?.(context)
                  }
                }
              `,
                        },
                    ],
                },
                {
                    messageId: 'interactionShouldBeAwaited',
                    data: { method: 'play' },
                    suggestions: [
                        {
                            messageId: 'fixSuggestion',
                            output: (0, ts_dedent_1.default) `
                export const FourthStory = {
                  play: async (context) => {
                    FirstStory.play(context)
                    SecondStory.play!(context)
                    await ThirdStory.play?.(context)
                  }
                }
              `,
                        },
                    ],
                },
            ],
        },
    ],
});
