"use strict";
/**
 * @fileoverview This rule identifies storybook addons that are invalid because they are either not installed or contain a typo in their name.
 * @author Andre "andrelas1" Santos
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
//------------------------------------------------------------------------------
// Requirements
//------------------------------------------------------------------------------
const utils_1 = require("@typescript-eslint/utils");
const no_uninstalled_addons_1 = __importDefault(require("../../../lib/rules/no-uninstalled-addons"));
const rule_tester_1 = __importDefault(require("../../utils/rule-tester"));
const path_1 = require("path");
jest.mock('fs', () => (Object.assign(Object.assign({}, jest.requireActual('fs')), { readFileSync: () => `
    {
      "devDependencies": {
        "@storybook/addon-essentials": "^6.5.9",
        "@storybook/addon-interactions": "^6.5.9",
        "@storybook/preset-create-react-app": "^6.5.9",
        "@storybook/addon-links": "^6.5.9",
        "storybook-addon-valid-addon": "0.0.1",
        "addon-without-the-prefix": "^0.0.1"
      }
    }
  ` })));
//------------------------------------------------------------------------------
// Tests
//------------------------------------------------------------------------------
rule_tester_1.default.run('no-uninstalled-addons', no_uninstalled_addons_1.default, {
    valid: [
        `
    export default {
      addons: [
        "@storybook/addon-links",
        "@storybook/addon-essentials",
        "@storybook/addon-interactions",
        "@storybook/preset-create-react-app"
      ]
    }
  `,
        `
    export default {
      addons: [
        "@storybook/addon-links",
        "@storybook/addon-essentials",
        "@storybook/addon-interactions",
        "@storybook/preset-create-react-app"
      ]
    } satisfies StorybookConfig
  `,
        `
     const config: StorybookConfig = {
        addons: [
        "@storybook/addon-links",
        "@storybook/addon-essentials",
        "@storybook/addon-interactions",
        "@storybook/preset-create-react-app"
        ]
      }
      export default config
  `,
        `
    export const addons = [
      "@storybook/addon-links",
      "@storybook/addon-essentials",
      "@storybook/addon-interactions",
    ]
  `,
        `
    module.exports = {
        addons: [
          "@storybook/addon-links",
          "@storybook/addon-essentials",
          "@storybook/addon-interactions",
        ]
      }
  `,
        `
    module.exports = {
        addons: [
          "storybook-addon-valid-addon/register",
          "addon-without-the-prefix/preset",
        ]
      }
  `,
        `
    module.exports = {
        addons: [
          "@storybook/addon-links",
          "@storybook/addon-essentials",
          "@storybook/addon-interactions",
          "storybook-addon-valid-addon",
        ]
      }
  `,
        `
    module.exports = {
        addons: [
          "@storybook/addon-links",
          "@storybook/addon-essentials",
          "@storybook/addon-interactions",
          "addon-without-the-prefix",
        ]
      }
  `,
        `
      module.exports = {
          addons: [
            "../my-local-addon",
            "../my-local-addon/index.cjs",
            "/Users/foo/my-local-addon/index.js",
            "/mount/foo/my-local-addon/index.js",
            "C:\\Users\\foo\\my-local-addon/index.js",
            "D:\\Users\\foo\\my-local-addon/index.js",
          ]
        }
    `,
        `
    module.exports = {
        addons: [
          {
            name: "@storybook/addon-links",
          },
          "@storybook/addon-essentials",
          "@storybook/addon-interactions",
          {
            name: "addon-without-the-prefix",
          },
          {
            name: "storybook-addon-valid-addon/register.js",
          },
        ]
      }
  `,
        {
            code: `
      module.exports = {
          addons: [
            "@storybook/addon-links",
            "@storybook/addon-essentials",
            "@storybook/addon-interactions",
            "@storybook/not-installed-addon",
          ]
        }
    `,
            options: [
                {
                    packageJsonLocation: '',
                    ignore: ['@storybook/not-installed-addon'],
                },
            ],
        },
    ],
    invalid: [
        {
            code: `
      module.exports = {
        addons: [
          "@storybook/addon-links",
          "@storybook/addon-essentials",
          "@storybook/addon-interactions",
          '@storybook/not-installed-addon'
        ]
      }
      `,
            errors: [
                {
                    messageId: 'addonIsNotInstalled', // comes from the rule file
                    // type: AST_NODE_TYPES.AssignmentExpression,
                    type: utils_1.AST_NODE_TYPES.Literal,
                    data: {
                        addonName: '@storybook/not-installed-addon',
                        packageJsonPath: `eslint-plugin-storybook${path_1.sep}`,
                    },
                },
            ],
        },
        {
            code: `
      module.exports = {
        addons: [
          "@storybook/addon-links",
          "@storybook/addon-essentials",
          "@storybook/addon-interactions",
          {
            name: '@storybook/not-installed-addon'
          }
        ]
      }
      `,
            errors: [
                {
                    messageId: 'addonIsNotInstalled', // comes from the rule file
                    type: utils_1.AST_NODE_TYPES.Literal,
                    data: {
                        addonName: '@storybook/not-installed-addon',
                        packageJsonPath: `eslint-plugin-storybook${path_1.sep}`,
                    },
                },
            ],
        },
        {
            code: `
      module.exports = {
        addons: [
          "@storybook/addon-links",
          {
            name: "@storybook/addon-esentials",
          },
          "@storybook/addon-interactions",
        ]
      }
      `,
            errors: [
                {
                    messageId: 'addonIsNotInstalled', // comes from the rule file
                    type: utils_1.AST_NODE_TYPES.Literal,
                    data: {
                        addonName: '@storybook/addon-esentials',
                        packageJsonPath: `eslint-plugin-storybook${path_1.sep}`,
                    },
                },
            ],
        },
        {
            code: `
      module.exports = {
        addons: [
          "@storybook/addon-links",
          "@storybook/adon-essentials",
          "@storybook/addon-interactions",
        ]
      }
      `,
            errors: [
                {
                    messageId: 'addonIsNotInstalled', // comes from the rule file
                    type: utils_1.AST_NODE_TYPES.Literal,
                    data: {
                        addonName: '@storybook/adon-essentials',
                        packageJsonPath: `eslint-plugin-storybook${path_1.sep}`,
                    },
                },
            ],
        },
        {
            code: `
      module.exports = {
        addons: [
          "@storybook/addon-links",
          "@storybook/addon-essentials",
          "@storybook/addon-interactions",
          "addon-withut-the-prefix",
          "@storybook/addon-esentials",
        ]
      }
      `,
            errors: [
                {
                    messageId: 'addonIsNotInstalled', // comes from the rule file
                    type: utils_1.AST_NODE_TYPES.Literal,
                    data: {
                        addonName: 'addon-withut-the-prefix',
                        packageJsonPath: `eslint-plugin-storybook${path_1.sep}`,
                    },
                },
                {
                    messageId: 'addonIsNotInstalled', // comes from the rule file
                    type: utils_1.AST_NODE_TYPES.Literal,
                    data: {
                        addonName: '@storybook/addon-esentials',
                        packageJsonPath: `eslint-plugin-storybook${path_1.sep}`,
                    },
                },
            ],
        },
        {
            code: `
      export default {
        addons: [
          "@storybook/addon-links",
          "@storybook/addon-essentials",
          "@storybook/addon-interactions",
          "addon-withut-the-prefix",
          "@storybook/addon-esentials",
        ]
      }
      `,
            errors: [
                {
                    messageId: 'addonIsNotInstalled', // comes from the rule file
                    type: utils_1.AST_NODE_TYPES.Literal,
                    data: {
                        addonName: 'addon-withut-the-prefix',
                        packageJsonPath: `eslint-plugin-storybook${path_1.sep}`,
                    },
                },
                {
                    messageId: 'addonIsNotInstalled', // comes from the rule file
                    type: utils_1.AST_NODE_TYPES.Literal,
                    data: {
                        addonName: '@storybook/addon-esentials',
                        packageJsonPath: `eslint-plugin-storybook${path_1.sep}`,
                    },
                },
            ],
        },
        {
            code: `
      export default {
        addons: [
          "@storybook/addon-links",
          "@storybook/addon-essentials",
          "@storybook/addon-interactions",
          "addon-withut-the-prefix",
          "@storybook/addon-esentials",
        ]
      } satisfies StorybookConfig
      `,
            errors: [
                {
                    messageId: 'addonIsNotInstalled', // comes from the rule file
                    type: utils_1.AST_NODE_TYPES.Literal,
                    data: {
                        addonName: 'addon-withut-the-prefix',
                        packageJsonPath: `eslint-plugin-storybook${path_1.sep}`,
                    },
                },
                {
                    messageId: 'addonIsNotInstalled', // comes from the rule file
                    type: utils_1.AST_NODE_TYPES.Literal,
                    data: {
                        addonName: '@storybook/addon-esentials',
                        packageJsonPath: `eslint-plugin-storybook${path_1.sep}`,
                    },
                },
            ],
        },
        {
            code: `
      const config: StorybookConfig = {
        addons: [
          "@storybook/addon-links",
          "@storybook/addon-essentials",
          "@storybook/addon-interactions",
          "addon-withut-the-prefix",
          "@storybook/addon-esentials",
        ]
      }
      export default config
      `,
            errors: [
                {
                    messageId: 'addonIsNotInstalled', // comes from the rule file
                    type: utils_1.AST_NODE_TYPES.Literal,
                    data: {
                        addonName: 'addon-withut-the-prefix',
                        packageJsonPath: `eslint-plugin-storybook${path_1.sep}`,
                    },
                },
                {
                    messageId: 'addonIsNotInstalled', // comes from the rule file
                    type: utils_1.AST_NODE_TYPES.Literal,
                    data: {
                        addonName: '@storybook/addon-esentials',
                        packageJsonPath: `eslint-plugin-storybook${path_1.sep}`,
                    },
                },
            ],
        },
        {
            code: `
        export const addons = [
          "../my-local-addon",
          "@storybook/addon-links",
          "@storybook/addon-essentials",
          "@storybook/addon-interactions",
          "addon-withut-the-prefix",
          "@storybook/addon-esentials",
        ]
      `,
            errors: [
                {
                    messageId: 'addonIsNotInstalled', // comes from the rule file
                    type: utils_1.AST_NODE_TYPES.Literal,
                    data: {
                        addonName: 'addon-withut-the-prefix',
                        packageJsonPath: `eslint-plugin-storybook${path_1.sep}`,
                    },
                },
                {
                    messageId: 'addonIsNotInstalled', // comes from the rule file
                    type: utils_1.AST_NODE_TYPES.Literal,
                    data: {
                        addonName: '@storybook/addon-esentials',
                        packageJsonPath: `eslint-plugin-storybook${path_1.sep}`,
                    },
                },
            ],
        },
    ],
});
